@@ -423,8 +423,6 @@ sub html_unescape {
     return $self->decode('utf8');
 }
 
-sub length { length shift->{bytestream} }
-
 sub md5_sum {
     my $self = shift;
     $self->{bytestream} = Digest::MD5::md5_hex($self->{bytestream});
@@ -453,6 +451,8 @@ sub quote {
     return $self;
 }
 
+sub size { length shift->{bytestream} }
+
 sub to_string { shift->{bytestream} }
 
 sub unquote {
@@ -500,6 +500,21 @@ sub url_unescape {
     return $self;
 }
 
+sub xml_escape {
+    my $self = shift;
+
+    # Replace "&", "<", ">", """ and "'"
+    for ($self->{bytestream}) {
+        s/&/&amp;/g;
+        s/</&lt;/g;
+        s/>/&gt;/g;
+        s/"/&quot;/g;
+        s/'/&apos;/g;
+    }
+
+    return $self;
+}
+
 # Helper for url_sanitize
 sub _sanitize {
     my $hex = shift;
@@ -556,8 +571,9 @@ Mojo::ByteStream - ByteStream
     $stream->url_escape;
     $stream->url_sanitize;
     $stream->url_unescape;
+    $stream->xml_escape;
 
-    my $length = $stream->length;
+    my $size = $stream->size;
 
     my $stream2 = $stream->clone;
     print $stream2->to_string;
@@ -622,10 +638,6 @@ the following new ones.
 
     $stream = $stream->html_unescape;
 
-=head2 C<length>
-
-    my $length = $stream->length;
-
 =head2 C<md5_sum>
 
     $stream = $stream->md5_sum;
@@ -642,6 +654,10 @@ the following new ones.
 
     $stream = $stream->quote;
 
+=head2 C<size>
+
+    my $size = $stream->size;
+
 =head2 C<to_string>
 
     my $string = $stream->to_string;
@@ -663,4 +679,8 @@ the following new ones.
 
     $stream = $stream->url_unescape;
 
+=head2 C<xml_escape>
+
+    $stream = $stream->xml_escape;
+
 =cut
