@@ -11,10 +11,9 @@ use overload '""' => sub { shift->to_string }, fallback => 1;
 use Mojo::ByteStream 'b';
 use Mojo::Parameters;
 use Mojo::Path;
-use Socket;
 
-__PACKAGE__->attr([qw/fragment host password port scheme user/]);
-__PACKAGE__->attr('base', default => sub { Mojo::URL->new });
+__PACKAGE__->attr([qw/fragment host port scheme userinfo/]);
+__PACKAGE__->attr(base => sub { Mojo::URL->new });
 
 # RFC 3986
 our $UNRESERVED = 'A-Za-z0-9\-\.\_\~';
@@ -30,29 +29,6 @@ sub new {
     return $self;
 }
 
-sub address {
-    my ($self, $address) = @_;
-
-    # Set
-    if ($address) {
-        $self->{address} = $address;
-        $self->host($address) unless $self->host;
-        return $self;
-    }
-
-    # Cached
-    return $self->{address} if $self->{address};
-
-    # Resolve
-    my $host = $self->host;
-    $self->{address} =
-        $host =~ /\b(?:\d{1,3}\.){3}\d{1,3}\b/
-      ? $host
-      : inet_ntoa(inet_aton($host));
-
-    return $self->{address};
-}
-
 sub authority {
     my ($self, $authority) = @_;
 
@@ -74,17 +50,19 @@ sub authority {
             $port = $2;
         }
 
-        $self->userinfo($userinfo);
-        $self->host($host);
+        $self->userinfo($userinfo ? b($userinfo)->url_unescape : undef);
+        $self->host($host         ? b($host)->url_unescape     : undef);
         $self->port($port);
 
         return $self;
     }
 
     # *( unreserved / pct-encoded / sub-delims )
-    my $host     = b($self->host)->url_escape("$UNRESERVED$SUBDELIM");
-    my $port     = $self->port;
-    my $userinfo = $self->userinfo;
+    my $host = b($self->host)->url_escape("$UNRESERVED$SUBDELIM");
+    my $port = $self->port;
+
+    # *( unreserved / pct-encoded / sub-delims / ":" )
+    my $userinfo = b($self->userinfo)->url_escape("$UNRESERVED$SUBDELIM\:");
 
     # Format
     $authority .= "$userinfo\@" if $userinfo;
@@ -97,6 +75,7 @@ sub authority {
 sub clone {
     my $self = shift;
 
+    # Clone
     my $clone = Mojo::URL->new;
     $clone->scheme($self->scheme);
     $clone->authority($self->authority);
@@ -104,6 +83,9 @@ sub clone {
     $clone->query($self->query->clone);
     $clone->fragment($self->fragment);
 
+    # Base
+    $clone->base($self->base->clone) if $self->{base};
+
     return $clone;
 }
 
@@ -235,38 +217,11 @@ sub to_string {
     $url .= lc "$scheme://" if $scheme && $authority;
     $url .= "$authority$path";
     $url .= "?$query" if @{$query->params};
-    $url .= "#$fragment" if $fragment->length;
+    $url .= "#$fragment" if $fragment->size;
 
     return $url;
 }
 
-sub userinfo {
-    my ($self, $userinfo) = @_;
-
-    # Set
-    if (defined $userinfo) {
-        my $user     = $userinfo;
-        my $password = '';
-
-        if ($user =~ /^([^\:]*)\:(.*)$/) {
-            $user     = $1;
-            $password = $2;
-        }
-
-        $self->user(b($user)->url_unescape->to_string);
-        $self->password(b($password)->url_unescape->to_string);
-
-        return $self;
-    }
-
-    # *( unreserved / pct-encoded / sub-delims / ":" )
-    my $user     = b($self->user)->url_escape("$UNRESERVED$SUBDELIM\:");
-    my $password = b($self->password)->url_escape("$UNRESERVED$SUBDELIM\:");
-
-    # Format
-    return $user ? "$user:$password" : undef;
-}
-
 1;
 __END__
 
@@ -284,8 +239,6 @@ Mojo::URL - Uniform Resource Locator
     );
     print $url->scheme;
     print $url->userinfo;
-    print $url->user;
-    print $url->password;
     print $url->host;
     print $url->port;
     print $url->path;
@@ -331,11 +284,6 @@ L<Mojo::URL> implements the following attributes.
     my $host = $url->host;
     $url     = $url->host('127.0.0.1');
 
-=head2 C<password>
-
-    my $password = $url->password;
-    $url         = $url->password('pass;w0rd');
-
 =head2 C<port>
 
     my $port = $url->port;
@@ -346,11 +294,6 @@ L<Mojo::URL> implements the following attributes.
     my $scheme = $url->scheme;
     $url       = $url->scheme('http');
 
-=head2 C<user>
-
-    my $user = $url->user;
-    $url     = $url->user('root');
-
 =head2 C<userinfo>
 
     my $userinfo = $url->userinfo;
@@ -366,11 +309,6 @@ following new ones.
     my $url = Mojo::URL->new;
     my $url = Mojo::URL->new('http://127.0.0.1:3000/foo?f=b&baz=2#foo');
 
-=head2 C<address>
-
-    my $address = $url->address;
-    $url        = $url->address('127.0.0.1');
-
 =head2 C<clone>
 
     my $url2 = $url->clone;
